/****************************************************************************/
/*              Red Flags and Red Herrings in the Execution Plans           */
/*                                                                          */
/*                         Dmitri V. Korotkevitch                           */
/*                        http://aboutsqlserver.com                         */
/*                          dk@aboutsqlserver.com                           */
/****************************************************************************/
/*						      Objects Creation                              */
/****************************************************************************/

set nocount on
go

use SQLServerInternals
go

if exists(select * from sys.views v join sys.schemas s on v.schema_id = s.schema_id where s.name = 'dbo' and v.name = 'vCustomersStateNumbers') drop view dbo.vCustomersStateNumbers;
if exists(select * from sys.procedures p join sys.schemas s on p.schema_id = s.schema_id where p.name = 'GetSalesByState1' and s.name = 'dbo') drop proc dbo.GetSalesByState1;
if exists(select * from sys.procedures p join sys.schemas s on p.schema_id = s.schema_id where p.name = 'GetSalesByState2' and s.name = 'dbo') drop proc dbo.GetSalesByState2;
if exists(select * from sys.tables t join sys.schemas s on t.schema_id = s.schema_id where t.name = 'Customers' and s.name = 'dbo') drop table dbo.Customers;
if exists(select * from sys.tables t join sys.schemas s on t.schema_id = s.schema_id where t.name = 'Orders' and s.name = 'dbo') drop table dbo.Orders;
go

/* Database schema and indexing are for demo purposes only. Table schema is logically incorrect for the shopping cart
system and suboptimal from indexing standpoint. They should not be used in production */

create table dbo.Customers
(
	CustomerID int not null identity(1,1),
	CustomerNum nvarchar(16) not null,
	CustomerName nvarchar(64) not null,
	CustomerAddress nvarchar(128) not null,
	CustomerCity nvarchar(64) not null,
	CustomerState nvarchar(32) not null,
	CustomerZipCode nvarchar(12) not null,

	constraint PK_Customers 
	primary key clustered(CustomerID)
);
go

create table dbo.Orders
(
	OrderID int not null identity(1,1),
	OrderNum varchar(32) not null,
	CustomerID int not null,
	OrderDate smalldatetime not null,
	Amount money not null,
	DeliveryInstructions nvarchar(64) null, -- increasing size of the row
	
	constraint PK_Orders
	primary key clustered(OrderID)
)
go

;with Addresses(ZipCode, State, City)
as
(
	select A.ZipCode, 'MA', A.City
	from (values('01742','Concord'),('02108','Boston')) A(ZipCode, City)
)
,N1(C) as (select 0 union all select 0) -- 2 rows
,N2(C) as (select 0 from N1 as T1 cross join N1 as T2) -- 4 rows
,N3(C) as (select 0 from N2 as T1 cross join N2 as T2) -- 16 rows
,IDs(ID) as (select row_number() over (order by (select NULL)) from N3)
,Data(CustomerNum,CustomerName,CustomerAddress,CustomerCity,CustomerState,CustomerZipCode)
as
(
	select 
		row_number() over (order by (select NULL)) 
		,A.State + ' Customer from ' + A.City + ' ' + convert(nvarchar(10),IDs.ID)
		,'Address in ' + A.State + A.City + ' ' + convert(nvarchar(10),IDs.ID)
		,A.City, A.State, A.ZipCode 
	from Addresses A cross join IDs
)
insert into dbo.Customers(CustomerNum, CustomerName, CustomerAddress, CustomerCity, CustomerState, CustomerZipCode)
	select convert(nvarchar(16),CustomerNum), CustomerName, CustomerAddress, CustomerCity, CustomerState, CustomerZipCode
	from Data;

;with Addresses(ZipCode, State, City)
as
(
	select A.ZipCode, 'FL', A.City
	from (values('33614','Tampa'),('32777','Jacksonville'),('33101','Miami'),
			('33301','Ft Lauderdale'),('32802','Orlando'),('32202','Tallahassee'),
			('34101','Naples'),('33830','Bartow'),('33499','Boca Raton'),
	--		('33040','Key West'),('33028','Hollywood'),('32635','Gainsville'),
			('32559','Pensacola'),('34284','Venice'),('34667','Hudson')) A(ZipCode, City)
)
,N1(C) as (select 0 union all select 0) -- 2 rows
,N2(C) as (select 0 from N1 as T1 cross join N1 as T2) -- 4 rows
,N3(C) as (select 0 from N2 as T1 cross join N2 as T2) -- 16 rows
,N4(C) as (select 0 from N3 as T1 cross join N3 as T2 cross join N2 as T3) -- 1024 rows
,N5(C) as (select 0 from N4 as T1 cross join N3 ) -- 16,384 rows
,IDs(ID) as (select row_number() over (order by (select NULL)) from N4)
,Data(CustomerNum,CustomerName,CustomerAddress,CustomerCity,CustomerState,CustomerZipCode)
as
(
	select 
		row_number() over (order by (select NULL)) 
		,A.State + ' Customer from ' + A.City + ' ' + convert(nvarchar(10),IDs.ID)
		,'Address in ' + A.State + A.City + ' ' + convert(nvarchar(10),IDs.ID)
		,A.City, A.State, A.ZipCode 
	from Addresses A cross join IDs
)
insert into dbo.Customers(CustomerNum, CustomerName, CustomerAddress, CustomerCity, CustomerState, CustomerZipCode)
	select convert(nvarchar(16),CustomerNum), CustomerName, CustomerAddress, CustomerCity, CustomerState, CustomerZipCode
	from Data;

declare
	@StartDate date = '2019-01-01'
	,@EndDate date = '2020-01-01';

;with Dates(OrderDate)
as
(
	select @StartDate
	union all 
	select dateadd(day,1,OrderDate) 
	from Dates
	where OrderDate < @EndDate
)
,Data(OrderNum,CustomerID,OrderDate)
as
(
	select
		row_number() over (order by (select NULL))
		,c.CustomerId
		,d.OrderDate
	from dbo.Customers c cross join Dates d
)
insert into dbo.Orders(OrderNum,CustomerID,OrderDate,Amount)
	select convert(nvarchar(16),OrderNum),CustomerID,OrderDate,OrderNum % 100 + 1
	from Data
option(maxrecursion 0);
go

create nonclustered index IDX_Customers_State
on dbo.Customers(CustomerState)
include(CustomerZipCode);

create nonclustered index IDX_Orders_CustomerId
on dbo.Orders(CustomerId)
include(OrderDate,Amount);

select count(*) as [Customer Count] from dbo.Customers;
select count(*) as [Order Count] from dbo.Orders;

select max(cnt), min(cnt), avg(cnt)
from 
	(
		select count(*) as cnt
		from dbo.Orders
		group by CustomerID
	) c;

dbcc show_statistics('dbo.Customers','IDX_Customers_State');



