/****************************************************************************/
/*                         SQL Server Internals v3                          */
/*                           Training Materials                             */
/*                                                                          */
/*                  Written by Dmitri V. Korotkevitch                       */
/*                      http://aboutsqlserver.com                           */
/*                        dk@aboutsqlserver.com                             */
/****************************************************************************/
/*                             Data Storage                                 */
/*                                Joins                                     */
/****************************************************************************/

use SQLServerInternals
go

if exists(select * from sys.tables t join sys.schemas s on t.schema_id = s.schema_id where s.name = 'dbo' and t.name = 'T1') drop table dbo.T1;
if exists(select * from sys.tables t join sys.schemas s on t.schema_id = s.schema_id where s.name = 'dbo' and t.name = 'T2') drop table dbo.T2;
go

create table dbo.T1
(
	IndexedCol int primary key,
	NonIndexedCol int,
);

create table dbo.T2
(
	IndexedCol int primary key,
	NonIndexedCol int,
);

;with N1(C) as (select 0 union all select 0) -- 2 rows
,N2(C) as (select 0 from N1 as T1 CROSS JOIN N1 as T2) -- 4 rows
,N3(C) as (select 0 from N2 as T1 CROSS JOIN N2 as T2) -- 16 rows
,N4(C) as (select 0 from N3 as T1 CROSS JOIN N3 as T2) -- 256 rows
,N5(C) as (select 0 from N4 as T1 CROSS JOIN N4 as T2) -- 65,536 rows
,N6(C) as (select 0 from N3 as T1 CROSS JOIN N5 as T2) -- 1,048,576 rows
,IDs(ID) as (select row_number() over (order by (select NULL)) from N6)
insert into dbo.T1(IndexedCol, NonIndexedCol)
	select ID, ID from IDs;

insert into dbo.T2(IndexedCol, NonIndexedCol)
	select IndexedCol, NonIndexedCol from dbo.T1;

set statistics time, io on

-- LOOP: Indexed vs. Nonindexed
-- Relatively fast: Index Seek on T2
select count(*)
from dbo.T1 inner loop join dbo.T2 on 
	T1.IndexedCol = T2.IndexedCol
where 
	T1.IndexedCol <= 50000;

-- Very slow: Multiple  scans

	select count(*)
	from dbo.T1 inner loop join dbo.T2 on 
		T1.IndexedCol = T2.NonIndexedCol
	where 
		T1.IndexedCol <= 50
option (querytraceon 8690); -- Disabling spool


-- LOOP vs Hash
-- Relatively fast: Index Seek on T2
select count(*)
from dbo.T1 inner loop join dbo.T2 on 
	T1.IndexedCol = T2.IndexedCol;

-- Fast: Single Table Scan on T2
select count(*)
from dbo.T1 inner hash join dbo.T2 on 
	T1.IndexedCol = T2.IndexedCol;

-- Fast: Single Table Scan on T2
select count(*)
from dbo.T1 inner hash join dbo.T2 on 
	T1.IndexedCol = T2.NonIndexedCol;
go

-- MERGE
-- Fast: Both inputs were sorter
select count(*)
from dbo.T1 inner merge join dbo.T2 on 
	T1.IndexedCol = T2.IndexedCol;

-- Sort operator in the plan
select count(*)
from dbo.T1 inner merge join dbo.T2 on 
	T1.IndexedCol = T2.NonIndexedCol;


-- Finally: How SQL Server Optimizes if
declare 
	@N int = 10;

select top (@N) T1.IndexedCol, T2.IndexedCol
from dbo.T1 join dbo.T2 on 
	T1.IndexedCol = T2.IndexedCol
option (loop join, hash join, optimize for (@N = 10));

select top (@N) * 
from dbo.T1 join dbo.T2 on 
	T1.IndexedCol = T2.IndexedCol
option (loop join, hash join, optimize for (@N = 100000));
go

-- Typical "issue"
update dbo.T2 set NonIndexedCol = 2 where IndexedCol >= 2;
create index IDX_T2_NonIndexedCol on dbo.T2(NonIndexedCol);
update dbo.T1 set NonIndexedCol = 2 where IndexedCol >= 2;
create index IDX_T1_NonIndexedCol on dbo.T1(NonIndexedCol);
go

dbcc show_statistics('dbo.T2',IDX_T2_NonIndexedCol);
go

declare 
	@V int = 2;

select count(*)
from dbo.T1 join dbo.T2 on 
	T1.NonIndexedCol = T2.NonIndexedCol
where 
	T1.IndexedCol between 101 and 125 and
	T1.NonIndexedCol = @V
option (loop join, hash join, optimize for (@V = 1));
go

